_me = frm_add_room.prototype;
function frm_add_room(){};

/**
 * @brief   Form for adding folder.
 * Form has three input objects:
 *    - text input for name of the folder (folder must
 *    match this regular expr. /^ *[a-zA-Z0-9#\.\_\-]+[ a-zA-Z0-9#\.\_\-]*$/).
 *    - select box for folder type specification (mail, contact, event, journal, note, task).
 *    - folder three.
 * 'OK' button is active only if the name of the folder match the regular expr. above.
 *
 * @param[in]  sAccountID  [string] Only folders from this account will be listed because of
 * moving folders between accounts is disabled.
 * @param[in]  sFolderID   [string] Optional. Forder with this ID will be preselected.
 *
 * @see  objects/frm_add_folder.xml
 * @see  inc/frm_add_folder.js
 */
_me.__constructor = function(sAccountID, sFolderID, aHandler){
	this._modal(true);

	var me = this;

	if (!Is.Defined(sAccountID)) return;

	this._title('POPUP_FOLDERS::ADD_ROOM');
	this._size(500,360,true);

	this._create('folder','obj_label','header')._value(Path.basename(sFolderID));

	// Create formular from template
	this._draw('frm_add_room', 'main');
	this._getAnchor('info').innerHTML = getLang('FORM_FOLDERS::CHAT_ROOM',[Path.basename(sFolderID)]);

	this.input_name._placeholder(getLang('FORM_FOLDERS::ROOM_PLACEHOLDER'));
	this.input_name._restrict('![/\\\\:\*\?\"\<\>\|\~]+','','^.{1,255}$');

	this.select_type._fill({
		I:'SHARING::PUBLIC',
		P:'SHARING::PRIVATE'
	});
	this.select_type._value('I');

	this.x_btn_ok._disabled(true);
	this.x_btn_ok._tabIndex();
	this.x_btn_cancel._tabIndex();

	function test (){
		var isError = !me.input_name.__check();
		me.x_btn_ok._disabled(isError);
	};
	this.input_name._onerror = function(){
		test();
	};
	this.input_name._onsubmit = function(){
		test();
		if (!me.x_btn_ok._disabled())
			me.x_btn_ok._onclick();
	};


	// Disable/Enable 'OK' button depending on reg. expr. of folder name. Initial
	// value of obj_input is empty, so initially 'OK' button should be disabled.
	// TODO dodelat restrictions
	// this.x_btn_ok._disabled(this.input_name.__check());

	// This function is triggered when 'OK' button is pressed
	this.x_btn_ok._onclick = function() {

		var sAccountName = sAccountID,
			sFolderName = sFolderID,
			aFolders = dataSet.get('folders',[sAccountID]);

		test();
		if (this._disabled())
			return false;

		if (sFolderName && !aFolders[sFolderName]){
            gui._create('alert','frm_alert','','','','','FORM_FOLDERS::CREATE_ERROR');
			return;
		}

		sFolderName = ((sFolderName) ? sFolderName+'/' : '') + me.input_name._value().trim();

		//Serch for already existing folder
		var bFound = false;
		for(var f in aFolders)
		    if (f.toLowerCase() == sFolderName.toLowerCase()){
		    	bFound = true;
		    	break;
		    }

		if (sFolderName && !bFound){
			var sFolderType = me.select_type._value(),
				bPrivate = 0;

			if (sFolderType == 'P'){
				sFolderType	= 'I';
				bPrivate = 1;
			}

			WMFolders.add({'name':sFolderName,'type':sFolderType,'aid':sAccountName, 'private':bPrivate},'folders','',aHandler);
			me._destruct();
		}
		else
			gui._create('alert','frm_alert','','',[function(){if (me.input_name) me.input_name._setRange(0,me.input_name._value().length);}],'','ALERTS::ROOM_EXIST',[Path.basename(sFolderName).escapeHTML()]);
	}
};