
/*
 *	Conference meeting window for both logged in and anonymous participants
 *
 *	Will start up sip if not online, join meeting as anonymous if a meeting id
 *	is supplied, otherwise log in and allow user to create a meeting and start it.
 *
 *	Actual conference communication is handled by wm_conference object and obj_sip
 *
 *	Martin Ekblom 2012-2014
 */

function frm_meeting() {};

frm_meeting.prototype.__constructor = function(conferenceId,groupId,callback) {
	var me = this;

	// Check for browser or plugin support
	if(!IceSIP.supported() || document.location.protocol!='https:') {
		var nortc = gui._create('no_webrtc','frm_confirm','','',[function(){
			window.open('https://www.icewarp.com/support/troubleshoot_webrtc/?' + buildURL({ishttps:(location.protocol.indexOf("https")===0?'1':'0'),lang:GWOthers.getItem('LAYOUT_SETTINGS','language')}));
		}],'SIP::SIP',document.location.protocol=='https:'?'SIP::NOWEBRTC':'SIP::NOTSECURE');
		nortc.x_btn_ok._value('ERROR::TROUBLESHOOT');
		addcss(nortc.x_btn_ok._main, 'help');
		this._destruct();
		return;
	}

	// If already waiting for media do not start meeting
	if(dataSet.get('sip',['media'])=='requesting') {
		gui._create('media_alert','frm_alert','','',[function(){}],'SIP::SIP','SIP::GETTINGMEDIA');
		this._destruct();
		return;
	}

	// If there is already SIP activity do not continue
	if(dataSet.get('sip',['activity'])) {
		gui._create('alert','frm_alert','','',[function(){}],'SIP::SIP','SIP::OCCUPIED');
		this._destruct();
		return;
	}

/*	this.performance = {
		app: new Date().valueOf(),
		java: new Date().valueOf(),
		server: new Date().valueOf(),
		sip: new Date().valueOf(),
		role: new Date().valueOf(),
		full: new Date().valueOf()
	}
	console.log('Meeting starting up');
*/
	this._defaultSize(-1,-1,900,600);
	this._title('CONFERENCE::MEETING');

	if(conferenceId)
		this.__meeting_id = conferenceId + '';

	if(groupId)
		this.__group_id = groupId;

	if(callback)
		this.__starthandler = callback;

	this.__state = 'preparing';
	this.__muted = false;
	this.__passive = false;
	this.__desktopsharing = false;
	this.__siprunning = false;
	this.__recording = null;
	this.__online = false;
	this.__invitebuffer = [];
	this.__thememapping = {};
	this.__theme = -1;
	this.__privateChat = false;
	this.__lastPrivateChat = 0;
	this.__participants = {};
	this.__next_theme = 0;
	this.__mikogo_app = /Windows (NT|XP|2000|ME)/.test(navigator.userAgent);
	this.__can_share = navigator.browser && navigator.browser.application=='Chrome' && location.protocol.indexOf("https")===0;

	// Check for installed plugin
	if(this.__can_share) {
		var image = new Image();
		image.onerror = function() {
			me.__can_share = false;
		}
	//	image.onload = function() {}
		image.src = 'chrome-extension://gpeicpibkllbejpnminnmecpjedmhhdg/icon.png';
	}

	this.__timer = {};
	this.__menudata = [];

	// Draw main window content
	this._draw('frm_meeting', 'main');

	// Create loader for showing during startup
	this._create('loader','obj_loader','box');
	this.loader._value(getLang('CONFERENCE::MEETING'));

	// Draw attendees list
	this._create('attendees','obj_list','participants');
	this.attendees._emptylabel(getLang('CONFERENCE::NOPARTICIPANTS'));
	this.attendees._scrollbar(this._getAnchor('participants'),this._getAnchor('participants').parentNode);
	this.attendees._fill([]);

	// Add chat room and chat input
	this.chatroom = this.main._create('chatroom','obj_chat','core');
	this.chat = this.main._create('chat','obj_text_dynamic_label_within','footer','chat');
	this.chat._disabled(true);

	// Add chat room and chat input
	this.options = this._create('options','obj_meeting_options','header');

	// Reset meeting dataset
	dataSet.add('meeting',['live'],false);
	dataSet.add('meeting',['id'],'');
	dataSet.add('meeting',['information'],{});
	dataSet.add('meeting',['desktop'],false);
	dataSet.obey(this,'_listenerMEETING','meeting');
	dataSet.obey(this,'_listenerSIP','sip',true);

	// Starting sip
	if(!gui.frm_main.sip) {
		gui.frm_main._create('sip','obj_sip');
		this._init();
	} else if(JsSIP) {
		if(gui.frm_main.sip && gui.frm_main.sip.phone && dataSet.get('sip',['state'])=='offline')
			gui.frm_main.sip.phone.sip.register();
		dataSet.add('meeting',['online'],true);
		this._init();
	}
}

frm_meeting.prototype._init = function() {
	var me = this;
//	console.log('Initialisation starting');

	if (gui.frm_main.sip)
		gui.frm_main.sip._add_callbeck([this,'_incoming']);

	this.__user = dataSet.get('main',['fullname']);
	this.__infos = {};	

	// Starting conference
	this._report('CONFERENCE::PREPARING');
	storage.library('wm_conference');
	this.conference = new wm_conference({object:this,method:this._update});
	if(this.__meeting_id)
		this.conference.setId(this.__meeting_id);

	this._add_destructor('__meeting_destruct');

	// Login organiser, create new meeting and start it
	if(sPrimaryAccountMEETING && !this.__meeting_id)
		this._organise();
	else
	// Get existing meeting and try to join
	if(!sPrimaryAccountMEETING)
		this._participate();
	// Existing meeting, participate or manage meeting if organiser
	else
		this._decide();


	// Check for return key for sending when typing chat message
	this.chat.__eIN.onkeydown = function(e) {
		e = e || window.event;
		var mess = me.chat._value();

		// Shift return for linebreak
		if(e.keyCode==13 && e.shiftKey)
			return true;

		// Return key, send message
		if(e.keyCode==13) {
			if(mess) {
				me.chat._value('');
				me.chat._focus(true);

				if(me.__privateChat && !dataSet.get('meeting',['participants',me.__privateChat])) {
					// Participant left meeting, inform user
					me._addchatline({message: getLang('CONFERENCE::LEFTCHAT')});
				} else {
					// Send chat message
					if(JsSIP) {
						gui.frm_main.sip._send(me.__userId,mess,me.__privateChat);
					} else {
						if(me.__privateChat)
							gui.frm_main.sip.conference.chat(mess,me.__privateChat);
						else
							gui.frm_main.sip.conference.send(mess);
					}
					// Add chat line to window
					me._addchatline({
						message: mess,
						from: me.__userId,
						to: me.__privateChat
					});

				}
			}
			e.returnValue = false;
			return false;
		}
	}

	// Remove private chat
	me.chat._main.onclick = function(e) {
		var elm = e ? e.target : event.srcElement;
		if(elm.nodeName=='EM') {
			me.chat._label('');
			me.__privateChat = false;
			me.__chatmodification();
		}
	}

	// Turn microphone off
	this.options.mute._onclick = function(e) {
		// For organisers also unmute on server
		if(me.bOrganiser && hascss(this._main,'server')) {
			me.conference.mute(me.__userId,false);
			// If not locally muted, take no more action
			if(!me.__muted) return;
		}
		// Mute or onmute locally - all attendees
		me.__muted = !me.__muted;
		gui.frm_main.sip.conference.mute(me.__muted);
		if(me.__muted)
			addcss(me.options.mute._main,'active');
		else
			removecss(me.options.mute._main,'active');
	}

	// Turn loudspeakers off
	this.options.silence._onclick = function(e) {
		me.__silenced = !me.__silenced;

		gui.frm_main.sip._silence(me.__silenced);

		if(me.__silenced)
			addcss(me.options.silence._main,'active');
		else
			removecss(me.options.silence._main,'active');
	}

	// Show info popup
	this.options.info._onclick = function(e) {
		// If info popup is already open, just focus it
		if(me.infofrm && !me.infofrm._destructed) {
			me.infofrm._focus();
			return;
		}

		// Create info popup
		me.infofrm = gui._create('info', 'obj_popup','','frm_meeting_info');
		me.infofrm._title('CONFERENCE::INFO');
		me.infofrm._size(500,350,1);
		me.infofrm._dockable(false);
		me.infofrm._resizable(false);
		me.infofrm._draw('frm_meeting_info','main');

		// Fill popup fields with current information
		var info = dataSet.get('meeting',['information']);
		info.id = dataSet.get('meeting',['id']);
		me.infofrm.id._value(info.id);
		me.infofrm.url._value(info.url);
		me.infofrm.phone._value(info.phone);
		me.infofrm.voip._value(info.sip);
		me.infofrm.callid._value(info.id.split('@')[0]);

		// Activate invite button
		me.infofrm.invite._onclick =  function(e) {
			NewMessage.compose({
				subject: getLang('CONFERENCE::INVITESUBJECT'),
				headers: {'X-ICEWARP-CONFERENCE': dataSet.get('meeting',['id'])},
				body: dataSet.get('meeting',['information','invitation']).replace(/\n/g,'<br/>')
			});
		}

		// Guest accounts are not allowed to invite
		if(sPrimaryAccountGUEST)
			me.infofrm.invite._destruct();

		//autoselect
		me.infofrm.callid._onclick = me.infofrm.voip._onclick = me.infofrm.phone._onclick = me.infofrm.url._onclick = me.infofrm.id._onclick = function(){
			this._select();
		};
	};

	// Toggle attendees visibility
	this.options.presence._onclick = function(e) {
		if(hascss(this._main,'active')) {
			removecss(this._main,'active')
			removecss(me._getAnchor('conference'),'presence');
		} else {
			addcss(this._main,'active')
			addcss(me._getAnchor('conference'),'presence');
		}

		me._onresize();
	}

	// Toggle chat visibility
	this.options.chat._onclick = function(e) {
		if(hascss(this._main,'active')) {
			removecss(this._main,'active')
			removecss(me._getAnchor('conference'),'chat');
		} else {
			addcss(this._main,'active')
			addcss(me._getAnchor('conference'),'chat');
			removecss(this._main,'alarm');
		}

		me._onresize();
	}

	this._onresize = function(e) {
		me.chatroom._scroll();

		var video = me._getAnchor('view').firstChild;
		if(video && video.nodeName=='VIDEO')
			video.onresize();
	}

	// Handling window closing
	this._onclose = function() {
		if(this._destructed)
			return true;

		// Do not allow close while waiting for device access
		if(dataSet.get('sip',['media'])=='requesting') {
			gui._create('media_alert','frm_alert','','',[function(){}],'SIP::SIP','SIP::GETTINGMEDIA');
			return false;
		}

		if (!this.cdialog || this.cdialog._destructed) {
			if(this.bOrganiser) {
				var organisers = false,
					managers = {'0':['','instruction']},
					length = 0,
					participants = dataSet.get('meeting',['participants']);
				for(var i in participants) {
					if(participants[i].email!=sPrimaryAccount && (participants[i].abilities && participants[i].abilities.permission || participants[i].role=='organizer'))
						organisers = true;
					if(participants[i].abilities && participants[i].abilities.control && participants[i].email!=sPrimaryAccount && participants[i].abilities.client) {
						managers[i] = [participants[i].name,''];
						length++;
					}
				}
				if(organisers) {
					// There are others who can moderate, organiser will be asked if role should be delegated
					gui._create('leave_end_confirm','frm_confirm_threestates', '','', [me,function(end) {
						if(end) { // Terminate conference
							me._create('loader','obj_loader','box');
							me.loader._value(getLang('CONFERENCE::ENDINFO'));
							me.__liquidate = true;
							me.conference.end();
						} else { // Leave conference
							me.conference.share(false);
							me._destruct();
						}
					}], 'MAIN_MENU::CONFERENCE','CONFERENCE::CLOSING','','CONFERENCE::END','CONFERENCE::LEAVE');
					gui.leave_end_confirm.x_btn_ok._main.className = 'obj_button ico cancel color2 big noborder simple';
					gui.leave_end_confirm.x_btn_cancel._main.className = 'obj_button ico ok color1 big noborder x_btn_right';
					gui.leave_end_confirm.x_btn_cancel2._main.className = 'obj_button big noborder simple ico cancel';
				} else if(length) {
					// There are other participants that could chair the meeting
					this.cdialog = gui._create('frm_confirm_select','frm_confirm_select', '','', [me,function(participant) {
						if(participant!='0') {
							me.conference.share(false);
							me.conference.delegate(participant);
							me._destruct();
						} else {
							me._create('loader','obj_loader','box');
							me.loader._value(getLang('CONFERENCE::ENDINFO'));
							me.__liquidate = true;
							me.conference.end();
						}
					}], 'MAIN_MENU::CONFERENCE','CONFERENCE::LEAVEOREND','',managers,'0');
				} else {
					// No other participants, only option end
					gui._create('frm_confirm','frm_confirm', '','', [me,function() {
						me._create('loader','obj_loader','box');
						me.loader._value(getLang('CONFERENCE::ENDINFO'));
						me.__liquidate = true;
						me.conference.end();
					}], 'MAIN_MENU::CONFERENCE','CONFERENCE::TERMINATE');
				}
			} else {
				// Common Participant, can only choose to leave meeting
				gui._create('frm_confirm','frm_confirm','','', [me,function() {
					me._destruct();
				}],'MAIN_MENU::CONFERENCE','CONFERENCE::LEAVING');
			}
		}

		return false;	// Alert will now handle closing
	}

	// translate ID -> User Name
	this.chatroom._translateUser = function(sUser){
		return (sUser == me.__userId ? getLang('CONFERENCE::ME') : (dataSet.get('meeting',['participants',sUser,'name']) || getLang('CONFERENCE::UNKNOWN')));
	};

	// select user
	this.chatroom._onclickuser = function(sUser){
		if (sUser == me.__userId || !(me.__privateChat = sUser)){
			me.__privateChat = false;
			me.chat._label('');
			me.chat._main.className = me.chat._main.className.replace(/theme[0-9]+/,'');
		}
		else {
			var user = dataSet.get('meeting',['participants',sUser]);
			if(user.name && user.colour) {
				me.chat._label(user.name, true);	
				if(me.chat._main.className.indexOf('theme')!=-1)
					me.chat._main.className = me.chat._main.className.replace(/theme[0-9]+/,'theme'+user.colour);
				else
					addcss(me.chat._main,'theme'+user.colour);
			}
		}

		me.__chatmodification();
	};

	// set user color, theme[0-10]
	this.chatroom._styleuser = function(sUser){
		return 'theme'+dataSet.get('meeting',['participants',sUser,'colour']);
	};
};


frm_meeting.prototype._chatinvite = function(aRecipients) {
	// If we have conference is running already, invite directly
	if(dataSet.get('meeting',['live'])) {
		if(gui.frm_main.im) gui.frm_main.im._send(aRecipients,getLang('CONFERENCE::CHATINVITE')+' '+dataSet.get('meeting',['information','url']),undefined,true);
	} else // otherwise remember to do later
		this.__invitebuffer.push.apply(this.__invitebuffer,aRecipients);
}



frm_meeting.prototype.__chatmodification = function() {
	this.attendees._selected(this.__privateChat || 'all');
	this.chat._focus();
}



frm_meeting.prototype._organise = function(bPromote) {
	var me = this;

/*	this.performance.role = new Date().valueOf()-this.performance.role;
	console.log('Decided role as organiser after',this.performance.role,'ms');
*/
	this.bOrganiser = true;

	if(!this.__passive)
		this.options.mute._disabled(false);

	this.options._create('menu','obj_hmenu','control','border');

	if(bPromote)
		this.bModerator = true;

	addcss(this._main,bPromote?'moderating':'organising');

	this.__menudata = [{
		title:'CONFERENCE::DESKTOP',
		title2: 'CONFERENCE::'+(!navigator.browser || navigator.browser.application!='Chrome' ? 'NOTCHROME' : location.protocol.indexOf("https")!==0 ? 'NOHTTPS' : !me.__can_share ? 'NOPLUGIN' : 'DESKTOPINFO'),
		"arg": 'share',
		css:"ico desktop",
		"disabled": dataSet.get('meeting',['desktop']) && this.bModerator || false
	},{
		title:'CONFERENCE::RECORD',
		title2: 'CONFERENCE::RECORDINFO',
		"arg": 'record',
		css:"ico record",
		"disabled": false
	}];
	// Guest user not allowed to invite others
	if(!sPrimaryAccountGUEST)
		this.__menudata.unshift({
			title:'CONFERENCE::INVITE',
			title2: 'CONFERENCE::INVITEINFO',
			"arg": 'invite',
			css:"ico invite",
			"disabled": false
		});

	// Organiser (and moderator) menu
	this.options.menu._onclick = function(e,elm,id,arg) {
		switch(arg) {
			case 'invite':
				NewMessage.compose({
					subject: getLang('CONFERENCE::INVITESUBJECT'),
					headers: {'X-ICEWARP-CONFERENCE': dataSet.get('meeting',['id'])},
					body: dataSet.get('meeting',['information','invitation']).replace(/\n/g,'<br/>')
				});
				break;
			case 'share':
				if(me.__desktopsharing !== null) {
					if(me.__can_share) {
						me.__menudata[1].css = "ico desktop pending";
						me.options.menu._fill(me.__menudata);
						if(me.__desktopsharing) {
							gui.frm_main.sip._unshare();
							me.bPresenter = false;
						} else {
							gui.frm_main.sip._share(true,function(share){
								if(me && !me._destructed) {
									me.__menudata[1].css = "ico desktop"+(share?' active':'');
									me.options.menu._fill(me.__menudata);
									me.conference.share(share);
								}
							});
							me.bPresenter = true;
						}
					}
					else { // IceWarp plugin is needed for screen sharing https://chrome.google.com/webstore/detail/icewarp-screen-sharing/gpeicpibkllbejpnminnmecpjedmhhdg/
						var lang = GWOthers.getItem('LAYOUT_SETTINGS','language');
						window.open('https://www.icewarp.com/support/troubleshoot_screensharing/?' + buildURL({ishttps:(location.protocol.indexOf("https")===0?'1':'0'),lang:lang=='pt'?'br':lang}));
					}
					me.__desktopsharing = null;
				}
				break;
			case 'record':
				me.__recording = !(dataSet.get('meeting',['recording']) || false);
				me.conference.record(me.__recording);
				me.__menudata.filter(function(item){
					return item.arg === 'record';
				})[0].css = "ico record pending";
				me.options.menu._fill(me.__menudata);
				break;
			case 'connect':
			//	me.conference.start();
				break;
		}
	}

	this.attendees._onclick = function(e,id) {
		var elm = e.target || e.srcElement;

		if(elm.tagName=='B') {
			// Click on mute button will mute participant
			elm = elm.parentNode;

		//	addcss(elm,'pending');

			var id = elm.getAttribute('id').match(/([0-9a-f]+|all)$/i)[1];
			if(id=="all")
				me.conference.muteall(me.__userId,!hascss(elm,'muted'));
			else
				me.conference.mute(id,!hascss(elm,'muted'));				
		} else {
			// Click on user will preselect that participant for private chat or public if to all
			var sel = this._selected()[0].substr(1);
			if(sel) {
				if(sel=='all') {
					me.chat._label('');
					me.__privateChat = false;
				} else {
					var user = dataSet.get('meeting',['participants',sel]) || {};
					me.chat._label(user.name || getLang('CONFERENCE::UNKNOWN'),true);
					if(me.chat._main.className.indexOf('theme')!=-1)
						me.chat._main.className = me.chat._main.className.replace(/theme[0-9]+/,'theme'+user.colour);
					else
						addcss(me.chat._main,"theme"+user.colour);
					me.__privateChat = sel;
				}
				me.__chatmodification();
			}
		}
	}

	this.attendees._oncontext = function(e,id,ids) {
		// Remove number sign added by oncontext from id
		if(id.indexOf('#')===0) id = id.substr(1);

		// Do not display context menu for All attendees
		if(id=="all") return false;

		var participant = dataSet.get('meeting',['participants',id]);
		var bSelf = participant.email==sPrimaryAccount;
		var bAllowed = participant.abilities && participant.abilities.control;
		var bClient = participant.abilities && participant.abilities.client;

		var aMenu = [
			{title: 'CONFERENCE::PRIVATECHAT', arg: 'chat', disabled: bSelf},
			{title: 'CONFERENCE::DELEGATE', arg: 'delegate', disabled: !bAllowed || bSelf || !bClient},
			{title: 'CONFERENCE::DISMISS', arg: 'dismiss', disabled: bSelf}
		];

		var cmenu = gui._create("cmenu","obj_context",'','',this);
			cmenu._fill(aMenu);
			cmenu._place(e.clientX,e.clientY);
			cmenu._onclick = function(e,elm,num,cmd) {
				switch(cmd) {
					case 'chat':
						var user = dataSet.get('meeting',['participants',id]) || {};
						me.chat._label(user.name,true);
						me.__privateChat = id;
						if(me.chat._main.className.indexOf('theme')!=-1)
							me.chat._main.className = me.chat._main.className.replace(/theme[0-9]+/,'theme'+user.colour);
						else
							addcss(me.chat._main,'theme'+user.colour);
						me.__chatmodification();
						me.chat._focus();
						break;
					case 'delegate':
						me.conference.delegate(id);
						break;
					case 'dismiss':
						me.conference.dismiss(id);
						break;
				}
			};
	}

	if (bPromote) {
		this.conference.manage(this.__call_id);
		this.options.menu._fill(this.__menudata);
	} else {
		if(this.__meeting_id)
			this.conference.login();
		else {
			var params = {desktop: 0};
			// If using a chat group, start meeting with all
			if(this.__group_id)
				params.folder = this.__group_id;
			// Create and run conference
			this.conference.create(params);
		}
	}
}

frm_meeting.prototype._participate = function() {
	var me = this;
/*	this.performance.role = new Date().valueOf()-this.performance.role;
	console.log('Decided role as participant after',this.performance.role, 'ms');
*/
	this.bOrganiser = false;

	this.__menudata = [];

	this.attendees._onclick = function(e,id) {
		var elm = e.target || e.srcElement;
		// Click on user will preselect that participant for private chat or public chat for all
		var sel = this._selected()[0].substr(1);
		if(sel) {
			if(sel=="all") {
				me.chat._label('');
				me.__privateChat = false;
			} else {
				var user = dataSet.get('meeting',['participants',sel]) || {};
				me.chat._label(user.name || getLang('CONFERENCE::UNKNOWN'),true);
				if(me.chat._main.className.indexOf('theme')!=-1)
					me.chat._main.className = me.chat._main.className.replace(/theme[0-9]+/,'theme'+user.colour);
				else
					addcss(me.chat._main,"theme"+user.colour);
				me.__privateChat = sel;
			}
			me.__chatmodification();
		}
	}

	if(dataSet.get('sip',['state'])=='online' && dataSet.get('meeting',['live'])) {
		this.__state = 'ongoing';	
		this._communicate();
	} else {
		this.__state = 'waiting';
	//	console.log('Waiting for meeting to start');
		this.conference.wait();
	}
}

frm_meeting.prototype._decide = function() {
	this.conference.get();
}

// Showing chat message
frm_meeting.prototype._addchatline = function(aData) {
	if (aData.from) {
		if(!hascss(this._getAnchor('conference'),'chat')) 
			addcss(this.options.chat._main,'alarm');

		this.chatroom._add(aData.from, aData.to, aData.message, (aData.from == this.__userId), new Date(), !aData.to);
	} else
		this.chatroom._system(aData.message);
};

frm_meeting.prototype._highlightSpeakers = function(speakers) {
	var p = dataSet.get('meeting',['participants']),
		s = {};
	if(p) {
		while(speakers.length)
			s[speakers.pop()] = true;
		for(var n in p)
			dataSet.add('meeting',['participants',n,'silent'],!s[n],true);
	}
	dataSet.update('meeting',['participants']);
}


// Message and error reporting
frm_meeting.prototype._report = function(sMessage,bError,bFatal) {
	var me = this;
	if (bError){
		if(!this.__erroralert || this.__erroralert._destructed)
			if(bFatal) {
				this.__erroralert = gui._create('alert', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', sMessage);
				this.__erroralert._onclose = function() {
					me._destruct();
					this._destruct();
				}
			} else
				this.__erroralert = gui._create('alert', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', sMessage);

	}
	else
	if (this.loader){
		if (sMessage)
			this.loader._value(getLang(sMessage));
	}
	else if(!this._destructed)
		this._alert = gui._create('alert', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', sMessage);

}

frm_meeting.prototype._update = function(sType,oData) {
	var me = this;
	// console.warn('Server performed: '+sType,oData);
	switch(sType) {
		case 'error':
			switch(parseInt(oData.code)) {
				// Mikogo errors, just inform
				case 114: case 115: case 116: case 117: case 136:
					this._report('CONFERENCE::MIKOGOERROR',true);
					gui.frm_main.sip._unshare();
					this.__desktopsharing = false;
					return false;
				// General errors from server which prevent full functionality (some of these may be better treated as fatal)
				case 101: case 102: case 105: case 108: case 110: case 112: case 123: case 127: case 128: case 129: case 131: case 133: case 134: 
					this._report('CONFERENCE::SERVERERROR',true);
					return false;
				// General errors related to permissions, non-fatal
				case 113: case 126:
					// If permission errors occur during shut down, just close meeting
					if(this.__liquidate) {
						this.__state = 'stopped';
						this._destruct();
					} else
						this._report('CONFERENCE::NEEDSPECIALPERMISSIONS',true);
					return false;
				case 132:
					// Ending screen sharing failed, silently ignore
					console.warn('Server reported insufficient permissions',oData.code);
					return false;
				case 1000:

					return false;
				// Treat as fatal error, suspend conference
				default:
					if(this.__timeInterval)
						clearInterval(this.__timeInterval);	// Stop timer

					this._addchatline({message: getLang('CONFERENCE::ABORTED')});

					if(oData.code && (oData.code==130 || oData.code>=118 && oData.code<=122))	// Network errors
						this._report('CONFERENCE::CONNECTIONERROR',true,true);
					if(oData.error && oData.code)	// Specific errors
						this._report('CONFERENCE::'+(wm_conference.error[oData.code]?wm_conference.error[oData.code].toUpperCase():'GENERALERROR'),true,true);

					return false;
			}
		case 'add':
			this._report('CONFERENCE::CREATED');
		case 'login':
			if(!this.__meeting_id)
				this.__meeting_id = oData.id;
			this.__menudata[0].disabled = false;
			this.__state = 'ready';
			if(this.__meeting_id)
				me.conference.start();	// Automatically start conference when we have a meeting id
			break;
		case 'start':
/*			this.performance.server = new Date().valueOf()-this.performance.server;
			console.log('Meeting started on server after',this.performance.server,'ms');
*/			this._report(this.__siprunning?'CONFERENCE::STARTEDCONNECT':'CONFERENCE::STARTEDNOJAVA');
			this.__state = 'ongoing';
			if(dataSet.get('sip',['state'])=='online')
				this._communicate();
			break;
		case 'modify':

			break;
		case 'changed':

			break;
		case 'startsharing':
			this.__desktopsharing = true;
			break;
		case 'stopsharing':
			this.__desktopsharing = false;
			break;
		case 'get':
			if(sPrimaryAccountMEETING && dataSet.get('meeting',['information','organizer']) == sPrimaryAccount) {
				this._organise();
			} else
				this._participate();
			break;
		case 'wait':
			if(dataSet.get('meeting',['live'])) { // This check is redundant because wait will not return until live :-/
/*				this.performance.server = new Date().valueOf()-this.performance.server;
				console.log('Meeting started on server after ',this.performance.server,'ms');
*/				this.__state = 'ongoing';
				if(dataSet.get('sip',['state'])=='online')
					this._communicate();
			} else
				this.conference.wait();
			break;
		case 'stop':
			this.__state = 'stopped';
			this._destruct();
			return;
	}

}

frm_meeting.prototype._communicate = function() {
	var me = this;
	var sip = gui.frm_main.sip;

	if(this.__online) return;

	if(sip && sip._alive()) {
		this.__online = true;

		var info = dataSet.get('meeting',['information']);

		if(gui.frm_main.im && this.__invitebuffer.length) {
			for(var i in this.__invitebuffer)
				gui.frm_main.im._send(this.__invitebuffer[i],getLang('CONFERENCE::CHATINVITE')+' '+info.url,undefined,true);
			this.__invitebuffer = [];
		}

		sip._join(info.sip);

		this._addtimer();
/*		this.performance.full = new Date().valueOf()-this.performance.full;
		console.log('Meeting is ready to start after',this.performance.full,'ms');
*/
		// PUT IT INTO TEMPLATE!
		var welcome =	getLang('CONFERENCE::WELCOME') + "\r\n" +
						getLang('CONFERENCE::INSTRUCT') + "\r\n" +
						getLang('CONFERENCE::ID') + ' ' + this.__meeting_id + "\r\n" +
						getLang('CONFERENCE::URL') + ' ' + info.url + "\r\n\r\n" +
						getLang('CONFERENCE::DIALIN') + "\r\n" + 
			(info.phone?getLang('CONFERENCE::PHONE') + ' ' + info.phone + "\r\n":'') +
						getLang('CONFERENCE::VOIP') + ' ' + info.sip + "\r\n\r\n" +
						getLang('CONFERENCE::RECAP');

		this._addchatline({message: welcome});
	} 
}

frm_meeting.prototype._addtimer = function(){
	var me = this;

	this.__timer.iterator = 0;
	this.__timer.timer = function(){
		var h = Math.floor(this.iterator/3600),
			m = Math.floor((this.iterator%3600)/60),
			s = ((this.iterator%3600)%60);

		me._title(getLang('CONFERENCE::MEETING') + ' - ' + (h?h+':':'')+(m<10?'0'+m:m)+':'+(s<10?'0'+s:s), true);

		this.iterator++;
	};
};

frm_meeting.prototype._incoming = function (aData){
	var me = this;
	var sip = gui.frm_main.sip;

	if(sip) {
		//console.log('Applet reports: '+aData.p1+' ('+aData.type+')');

		switch(aData.type) {
			case 'Error':
				// Errors are handled in obj_sip (this will never happen)
				console.warn('Error occured!',aData);
				break;

			case 'ConferenceStarted':
				if(typeof aData.p2=='object') {
					// Connected to conference, get call id
					this.__call_id = aData.p2.callid;
					dataSet.add('meeting',['callid'],aData.p2.callid,false);
					dataSet.add('meeting',['online'],true);

					// Keep on checking for changes to conference
					this.conference.monitor();

/*					this.performance.sip = new Date().valueOf()-this.performance.sip;
					console.log('Voice communication started after',this.performance.sip,'ms');
*/
					// Add timer for call duration
					if(this.__timer) {
					//	this.__timer.iterator = 0;

						if(this.__timeInterval)
							clearInterval(this.__timeInterval);

						this.__timeInterval = setInterval(function(){
							try{me.__timer.timer();}catch(e){clearInterval(me.__timeInterval);}
						}, 1000);
					}

					// Remove startup messages and show menu
					if (this.loader)
						this.loader._destruct();

					if (sip._passive()) {
						this.__passive = true;
						this.options.mute._disabled(true)
					}

					if (this.options.menu)
						this.options.menu._fill(this.__menudata);

					if(this.__starthandler) {
						this.__starthandler.push([this.__meeting_id]);
						executeCallbackFunction(this.__starthandler);
						this.__starthandler = null;
					}

				} else {
					// Conference call could not be established
					this._report('CONFERENCE::CONNECTFAILURE',true);
				}
				break;
			case 'ConferenceInformation':
				if(aData.p1=="text/plain" && aData.p2.indexOf('Speakers: ')===0) {
					var speakers = aData.p2.substr(10);
					speakers = speakers ? speakers.split(";") : [];
					this._highlightSpeakers(speakers);
				}
				break;
			case 'IncomingConferenceMessage':
				this._addchatline({
					from: aData.p1,
					message: aData.p2
				});
				break;
			case 'IncomingParticipantMessage':
				if(this.__userId)
					this._addchatline({
						from: aData.p1,
						to: this.__userId,
						message: aData.p2
					});
				break;
			case 'ConferenceCanceled':
				this.__passive = false;
				switch(aData.p2) {
					case 'ICE failed':
						this._report('CONFERENCE::CONNECTFAILURE',true);
						break;
					default:
						this._report('CONFERENCE::CANCELLED',true);
				}
				this._destruct();
				break;
			case 'ConferenceFinished':
				this.__passive = false;
				this._report('CONFERENCE::CLOSED',true);
				this._destruct();
				return;
		}

		if(this.__timeInterval && (aData.type == 'ConferenceCanceled' || aData.type == 'ConferenceFinished' || aData.type == 'Notification' && aData.p2 =='offline'))
			clearInterval(this.__timeInterval);	// Stop timer

	}
	
}

frm_meeting.prototype.__update = function(sDataSet,aProperties) {
	var sType, sItem;
	var me = this;

	if(aProperties && aProperties[0]) {
		sType = aProperties[0];
		if(aProperties[1]) sItem = aProperties[1];
	}

	// SIP online/offline handling
	if(sDataSet=='sip') {
		// If meeting has not started yet because of offline state, start it now
		if(sType=='state' && dataSet.get('sip',['state'])=='online' && !this.__online && this.__state=='ongoing')
			this._communicate();

		return;
	}

	// Changes to meeting
	switch(sType) {
		case 'live':
			// Close meeting when server stops it
			if(dataSet.get('sip',['activity'])=="Meeting" && !dataSet.get('meeting',['live'])) {
				this._report('CONFERENCE::CLOSED',true);
				this._destruct();
			}
			break;
		case 'id':
			var id = dataSet.get('meeting',['id']);
			this.__infos.id = id;
			break;
		case 'desktop':
			var share = dataSet.get('meeting',['desktop']);
			if(!this.bPresenter) {
					me._getAnchor('view').innerHTML = '';
					if(share)
						gui.frm_main.sip._share(false,function(video){
							if(video) {
								me.__video_size = null;
								addcss(me._main,'video');
								video.style.display = video.nodeName=='OBJECT' ? 'block' : 'none';
								me._getAnchor('view').appendChild(video);

								video.onwaiting = function(e) {
									if(!me._destructed) {
										// Waiting for video to start playing
										me._create('waiting','obj_loader','view');
										me.waiting._value(getLang('CONFERENCE::WAITVIDEO'));
									}
								}

								video.onplaying = function(e) {
									// Video started playing
									if(me.waiting)
										me.waiting._destruct();

									// Call resize to make video right size
									this.onresize();
								}

								if(navigator.browser && navigator.browser.application=='Firefox')
									// Temporary hack because FF does not set dimensions until after video started playing
									video.ontimeupdate = function(e) {
										if(me.__video_size && this.videoWidth && (me.__video_size.width==0 || me.__video_size.width!=this.videoWidth))
											this.onresize();
									}

								if(video.nodeName=='OBJECT') {
									// Resize interval for Plugin
									me.__resizeIntervalForIE = setInterval(function(){
										video.width = video.videoWidth = video.videoTrackWidth;
										video.height = video.videoHeight = video.videoTrackHeight;
										if(video.videoWidth && video.videoHeight)
											video.onresize.call(video,{});
									},1000);
								}

								video.onresize = function(e) {
									if(this.parentNode) {
										me.__video_size = {width: this.videoWidth, height: this.videoHeight};
										if(this.videoWidth)
											this.style.maxWidth = this.videoWidth+'px';
										if(this.videoHeight)
											this.style.maxHeight = this.videoHeight+'px';
										this.style.width = '98%';
										this.style.height = this.nodeName=='OBJECT'?'inherit':'auto';
										this.style.margin = '5px auto 0';
										this.style.display = 'block';
										var space = document.msFullscreenElement ? screen.height : this.parentNode.clientHeight;
										if(this.clientHeight<space)
											this.style.margin = ((space-this.clientHeight)/2)+'px auto 0';
										else if(this.videoHeight && this.videoHeight>space) {
											this.style.width = 'auto';
											this.style.height = (space-10)+'px';
										}
									}
								}

								video.onclick = function(e) {
									me._fullscreen(video);
								}
								
								video.play();
							} else
								removecss(me._main,'video');

							// Enable or disable sharing button
							if(me.bOrganiser || me.bModerator) {
								me.__menudata[1].disabled = !!video;
								me.options.menu._fill(me.__menudata);
							}
						});
					else {
						removecss(me._main,'video');
						if(me.__resizeIntervalForIE)
							clearInterval(me.__resizeIntervalForIE);
						gui.frm_main.sip._unshare();
					}
			}
			break;
		case 'information':
		//	console.log('Received meeting information');
			var info = dataSet.get('meeting',['information']);
			this.__infos.sip = info.sip;
			this.__infos.url = info.url;
			this.__infos.phone = info.phone;
			break;
		case 'speaker':
			// Mute all on or off			
			break;
		case 'participants':
		//	console.log('Received participants list');
			var pl = dataSet.get('meeting',['participants']) || {};
			var bPrivateChatLeft = false;
			// Save user_id for yourself and check if you became organiser
			for(var p in pl) {
				// If name is missing, use first part of email (external sip calls have no name)
				if(!pl[p].name && pl[p].email)
					pl[p].name = pl[p].email.split('@')[0];

				// Upgrade user to manager of the conference if promoted
				if(pl[p].role=='manager' && pl[p].email==sPrimaryAccount && !this.bOrganiser)
					this._organise(true);

				// Note id for this user and update muted state
				if(pl[p].email==sPrimaryAccount || GWOthers.getItem('SIP','user') && pl[p].email==GWOthers.getItem('SIP','user') && GWOthers.getItem('SIP','external')>0 && (GWOthers.getItem('RESTRICTIONS', 'disable_esip') || 0)<1 || sPrimaryAccountGUEST && pl[p].email==dataSet.get('main',['user'])) {
					this.__userId = p;

					if(pl[p].abilities.muted) {
						// Show server mute
						addcss(me.options.mute._main,'server');
						// Disable unmute for normal participant
						if(!this.bOrganiser && !this.options.mute._disabled())
							this.options.mute._disabled(true);
					} else {
						removecss(me.options.mute._main,'server');
						// Enable mute for active participant
						if(!this.__passive && !this.bOrganiser)
							this.options.mute._disabled(false);
					}
				}

			}
			// Compare incoming list to current, anyone left or changed?
			for(var p in this.__participants) {
				if(p in pl) {
					// Keep colour theme for user as assign from beginning
					pl[p].colour = this.__participants[p].colour;
					// If role changed, notify participants
					if(this.__participants[p].role!=pl[p].role && pl[p].role=='manager')
						this._addchatline({message: pl[p].name+' '+getLang('CONFERENCE::PROMOTED')});
				} else if(p!=this.__userId) {
					// If participant left, notify others
					this._addchatline({message: this.__participants[p].name+' '+getLang('CONFERENCE::LEFT')});
					// If in private chat with participant that just left
					if(this.__privateChat && p==this.__privateChat)
						bPrivateChatLeft = true;
				}
			}

			// Compare incoming list to current, anyone joined?
			for(var p in pl)
				if(!(p in this.__participants) && p!=this.__userId) {
					// Add message for newly arrived participant
					this._addchatline({message: pl[p].name+' '+getLang('CONFERENCE::CAME')});
					// Assign user theme 1 - 10
					pl[p].colour = this.__next_theme++ == 10 ? this.__next_theme = 1 : this.__next_theme;
				} else if(p==this.__userId)
					pl[p].colour = 0;	// theme 0 always yourself

			// Update participant lists
			var lista = {},
				listb = {},
				length = 0,
				bAllMuted = true;

			for(var p in pl) {
				// If the user is not in the list something is wrong
			/*	if(!this.__userId) {
					gui._create('alert', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', 'CONFERENCE::MISSINGUSERID');
					this._destruct();
					return;
				}
*/
				// Disregard yourself
				if(this.__userId==p)
					continue;

				// If any attendee is muted then all are not
				if(!pl[p].abilities.muted)
					bAllMuted = false;

				// Add to list of participants
				var css = ['theme'+pl[p].colour];
				if (pl[p].role) css.push(pl[p].role);
				if (pl[p].abilities.muted) css.push('muted');
				if (this.__selected_participant==p) css.push('selected');
				if (!pl[p].silent) css.push('noise');

				var html = (this.bOrganiser?'<b></b>':'') +'<em>'+(pl[p].name?pl[p].name.escapeXML():pl[p].email.escapeXML())+'</em>' + (pl[p].role!='participant'?'<br>'+getLang('CONFERENCE::'+pl[p].role.toUpperCase()):'');
				
				if (pl[p].role=='organizer' || pl[p].role=='manager')
					lista[p] = {html: html, css: css.join(' '), tooltip: pl[p].name.length>22?pl[p].name:''};
				else
					listb[p] = {html: html, css: css.join(' '), tooltip: pl[p].name.length>22?pl[p].name:''};

				length++;
			}
			
			if (length) {
				// Add All attandees to list
				var list = {
					all: {html: (this.bOrganiser?'<b></b>':'') +'<em>'+getLang('CONFERENCE::ALLATTENDEES')+'</em>', css: 'all'+(dataSet.get('meeting',['speaker'])?' muted':'')}
				}
				// Add organisers to the list
				for(var p in lista)
					list[p] = lista[p];
				// Add participants to the list
				for(var p in listb)
					list[p] = listb[p];

				// Fill list with all participants
				this.attendees._fill(list);
			} else
				// Empty list if only you are left
				this.attendees._fill([]);



			// Enable or disable chat if not live or no other participants
			this.chat._disabled(length<1 || !dataSet.get('meeting',['live']) || bPrivateChatLeft);

			// Save list to compare with if changes
			this.__participants = {};
			for(var p in pl)
				this.__participants[p] = pl[p];

			break;
		case 'recording':
			// Only organiser can affect recording
			var rec = dataSet.get('meeting',['recording']);
			if(this.bOrganiser) {
				// Indicate recording status
				this.__menudata[2].css = 'ico record' + (rec ? ' active' : '');
				if(this.__call_id)
					this.options.menu._fill(this.__menudata);
				// Show recording terminated message
				if(this.__recording!=null && rec==false) {
					gui._create('recorded', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', 'CONFERENCE::FINISHED');
					this.__recording = null;
				}
			}
			break;
	}
}

frm_meeting.prototype.__meeting_destruct = function() {
	// Show recording termination message if leaving while recording
	if(this.bOrganiser && this.__recording)
		gui._create('recorded', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', 'CONFERENCE::FINISHED');

	// Remove meeting listeners
	dataSet.disobey(this,'_listenerSIP');
	dataSet.disobey(this,'_listenerMEETING');

	// Clear resizing interval for Plugin
	if(this.__resizeIntervalForIE)
		clearInterval(this.__resizeIntervalForIE);

	// Send leave request
	if (gui.frm_main.sip) try {
		if(dataSet.get('meeting',['live']))
			// Just leave meeting by ending calls for audio and sharing
			gui.frm_main.sip._leave();
		else
			// Conference has ended, server will end audio call, just end sharing
			gui.frm_main.sip._unshare();
	} catch(e) {
		// JsSIP will throw an exception if trying to terminate an already ended ended call
	}

	// Close separate sharing window if open
	if(this.win && this.win.close)
		this.win.close();

	// Clear meeting dataset
	dataSet.remove('meeting');
}

/** DOCKING **/
frm_meeting.prototype._ondock = function (){
	return getLang('CONFERENCE::MEETING');
};

frm_meeting.prototype._onundock = function (){
	this.chat._focus(true);
	this.chatroom._scroll();
	return true;
};