/*
*/
_me = obj_websocket.prototype;
function obj_websocket(){};

_me.__constructor = function(sURL){

	//Default URL
	this.__sURL = sURL || (location.protocol == 'https:'?'wss:':'ws:') + '//' + location.hostname + (location.port?':'+location.port:'');

	// Communication protocol (Optional)
	this.__sProtocol = 'xmpp';
	this.__buffer = [];

	this._add_destructor('__destruct');

	this._open();
};

// CONNECTING	0 	The connection is not yet open.
// OPEN 		1 	The connection is open and ready to communicate.
// CLOSING 		2 	The connection is in the process of closing.
// CLOSED 		3 	The connection is closed or couldn't be opened.
_me._open = function(){

	if (!this.__socket && WebSocket){

		//console.log('obj_websocket','Connecting', this.__sURL);

		var me = this;

		try{
			this.__socket = new WebSocket(this.__sURL, this.__sProtocol);
		}
		catch(r){
			console.log("obj_websocket", "Your browser does not support WebSocket object", r);
			return;
		}

		this.__socket.onopen = function (e) {

			//console.log('#OPEN', e);

			if (me._onopen)
				me._onopen();

			me.__exeEvent('onopen',e,{"owner":me});

			me._buffer_flush();
		};

		this.__socket.onmessage = function(e){

			//console.log('#MESSAGE', e.data);

			if (me._onmessage)
				me._onmessage(e.data);

			me.__exeEvent('onmessage', e, {"owner":me});
		};

		this.__socket.onerror = function(e){

			console.log('websocket','ERROR', e);

			if (me._onerror)
				me._onerror();

			me.__exeEvent('onerror',e,{"owner":me});
		};

		//Error codes 	http://tools.ietf.org/html/rfc6455#section-7.4
		//				https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent#Status_codes
		//1013 	Try Again Later
		//1000	Normal
		this.__socket.onclose = function(e){

			console.log('websocket','CLOSE', e);

			if (me._onclose)
				me._onclose(e);

			me.__exeEvent('onclose',e,{"owner":me});

			delete me.__socket;

			//wasClean doesnt work with icewarp correctly
			if (e.wasClean == false){
				if (gui.__online)
					setTimeout(function(){ me._open(); }, (e.code == 1013?10000:5000));
				else
					gui._obeyEvent('online', [function(){
						me._open();
						return false;
					}]);
			}
		};
	}
};

/* code 1000 is not propagated correctly to onclose() */
_me._close = function(){
	if (this.__socket && this.__socket.readyState < 2)
		this.__socket.close(1000, 'bye bye');
};

_me._getState = function(){
	return this.__socket?this.__socket.readyState:3;
};

_me._send = function(sData){

	if (!this.__socket || this.__socket.readyState > 1){
		this.__buffer_queue(sData);
		this._open();
	}
	else
	if (this.__socket.readyState == 1){
		try{
			this.__socket.send(sData);
			return true;
		}
		catch(r){
			console.log("obj_websocket", "Unable to send request:", sData);
		}
	}
	else
		this.__buffer_queue(sData);

	return false;
};
	// PUBLIC
	_me._sendArray = function(aData, bPreserveCase){
		this._send(XMLTools.Arr2Str(aData,'',bPreserveCase));
	};
	_me._sendJSON = function(aData){
		this._send(JSON.stringify(aData));
	};
	_me._sendText = function(sData){
		this._send(sData);
	};

// BUFFER
_me.__buffer_queue = function(sXML) {
	this.__buffer.push(sXML);
};

_me._buffer_flush = function() {
	while(this.__buffer.length)
		this._send(this.__buffer.shift());
};

_me._buffer_clear = function() {
	this.__buffer = [];
};

_me._buffer_size = function() {
	return this.__buffer.length;
};

_me.__destruct = function(){
	this._buffer_clear();

	//kill onclose
	delete this._onclose;

	this._close();
};